require_relative '../../../../spec/spec_helper'

describe 'Create index parsing' do
  let(:parser) { DDLParser::DDL::DB2::Parser.new }
  context 'create index 1' do
    it 'parses create_unique_index' do
      sql = <<EOF
CREATE UNIQUE INDEX COST_INVOICE_1 ON COST_INVOICE
 (DOC_NO, DOC_NO_2 )
EOF
      begin
        result = parser.parse(sql.downcase)
      rescue Parslet::ParseFailed => error
        puts error.cause.ascii_tree
      end

      result.count.should == 3
      result[0].should include(:operation, :object_property, :object_type, :index_name)
      result[0].should include(:operation => 'create')
      result[0].should include(:object_property => 'unique')
      result[0].should include(:object_type => 'index')
      result[1].should include(:field)
    end
    it 'parses create_unique_index with one column' do
      sql = <<EOF
CREATE UNIQUE INDEX COST_INVOICE_1 ON COST_INVOICE
 (DOC_NO)
EOF
      begin
        result = parser.parse(sql.downcase)
      rescue Parslet::ParseFailed => error
        puts error.cause.ascii_tree
      end

      result.count.should == 6
      result.should include(:operation => 'create')
      result.should include(:object_property => 'unique')
      result.should include(:object_type => 'index')
      result.should include(:index_name => 'cost_invoice_1')
      result.should include(:table_name => 'cost_invoice')
      result.should include(:field => 'doc_no')
    end
    it 'parses create_unique_index with two columns' do
      sql = <<EOF
CREATE UNIQUE INDEX COST_INVOICE_1 ON COST_INVOICE
 (DOC_NO, DOC_NO_1)
EOF
      begin
        result = parser.parse(sql.downcase)
      rescue Parslet::ParseFailed => error
        puts error.cause.ascii_tree
      end

      result.count.should == 3
      result.is_a?(Array).should == true

      result[0].should include(:operation => 'create')
      result[0].should include(:object_property => 'unique')
      result[0].should include(:object_type => 'index')
      result[0].should include(:index_name => 'cost_invoice_1')
      result[0].should include(:table_name => 'cost_invoice')
      result[1].should include(:field => 'doc_no')
      result[2].should include(:field => 'doc_no_1')
    end
  end
  context 'element list index' do
    it 'parses one element' do
      sql = 'deptno asc'
      parser.element_list_index.parse(sql).should == {:field=> 'deptno',:sort_type=> 'asc'}
    end
    it 'parses two element' do
      sql = 'field1, field2'
      parser.element_list_index.parse(sql).should == [{:field=> 'field1'},
                                                      {:field=> 'field2'}]

    end

    it 'parses with new lines' do
      sql = <<EOF
DEPTNO desc,
DEPTNAME asc
EOF
      parser.element_list_index.parse(sql.downcase).should == [{:field=> 'deptno',:sort_type=> 'desc'},
                                                               {:field=> 'deptname',:sort_type=> 'asc'}]
    end
  end
  context 'index column definition' do
    it 'parses two element' do
      sql = 'field1'
      parser.index_column_definition.parse(sql).should == {:field=> 'field1'}


    end

    it 'parses with new lines' do
      sql = <<EOF
DEPTNO descending
EOF
      parser.index_column_definition.parse(sql.downcase).should == {:field=> 'deptno',:sort_type=> 'descending'}

    end
  end
  context 'reverse scan' do
    it 'parses reverse scans option' do
      sql = 'allow reverse scans'
    end
    it 'parses index_option with reverse scans' do
      sql = 'allow reverse scans'
      parser.index_option.parse(sql)[:index_option].should include(:reverse_scans_property => 'allow reverse scans')
    end
  end
  context 'create_index_statement' do
    it 'create_index_statement_1_unique_1' do
      sql = <<EOF
CREATE UNIQUE INDEX TEST_INDEX
EOF
      parser.create_index_statement.parse(sql.downcase.strip).should include(:operation=>'create')

    end
    it 'create_index_test_statement_unique_1_a' do
      parser.create_index_statement.parse('create unique index test_index').should == {:operation=>'create',:object_property=>'unique',:object_type=>'index',:index_name=>'test_index'}
    end
    it 'create_index_test_statement_unique_1_b' do
      parser.create_index_statement.parse('create unique index test_index').should include(:operation=>'create')
    end
    it 'create_index_statement_unique_3' do
      parser.create_index_statement.parse('create index cost_invoice_1').should == {:operation=>'create',:object_type=>'index',:index_name=>'cost_invoice_1'}
    end
    it 'create_index_statement_unique_4' do
      sql = <<EOF
CREATE UNIQUE INDEX COST_INVOICE_1
EOF
      parser.create_index_statement.parse(sql.downcase).should == {:operation=>'create',:object_property=>'unique',:object_type=>'index',:index_name=>'cost_invoice_1'}

    end
  end
  context 'index_table' do
    it 'index_table_example' do
      sql = <<EOF
ON COST_INVOICE
EOF
      parser.index_table.parse(sql.downcase).should == {:table_name => 'cost_invoice'}

    end
  end
  context 'full index example with reverse scan option' do
    it 'with reverse scan 1' do
      sql = <<EOF
CREATE UNIQUE INDEX COST_INVOICE_1
        ON COST_INVOICE
                (DOC_NO)
        ALLOW REVERSE SCANS
EOF
      begin
        result = parser.parse(sql.downcase)
      rescue Parslet::ParseFailed => error
        puts error.cause.ascii_tree
      end
      result.count.should == 7
      result.should include(:operation => 'create')
      result.should include(:object_property => 'unique')
      result.should include(:object_type => 'index')
      result.should include(:index_name => 'cost_invoice_1')
      result.should include(:table_name => 'cost_invoice')
      result.should include(:field => 'doc_no')
      result[:index_option].should include(:reverse_scans_property => 'allow reverse scans')

    end
    it 'without reverse scan' do
      sql = <<EOF
CREATE UNIQUE INDEX COST_INVOICE_1
        ON COST_INVOICE
                (DOC_NO)
EOF
      begin
        result = parser.parse(sql.downcase)
      rescue Parslet::ParseFailed => error
        puts error.cause.ascii_tree
      end
      result.count.should == 6
      result.should include(:operation => 'create')
      result.should include(:object_property => 'unique')
      result.should include(:object_type => 'index')
      result.should include(:index_name => 'cost_invoice_1')
      result.should include(:table_name => 'cost_invoice')
      result.should include(:field => 'doc_no')

    end
    it 'with reverse scan 2' do
      sql = <<EOF
CREATE INDEX PL_TRIGGER_I2
        ON PL_TRIGGER
                (UNIQUE_NO)
        ALLOW REVERSE SCANS
EOF
      begin
        result = parser.parse(sql.downcase)
      rescue Parslet::ParseFailed => error
        puts error.cause.ascii_tree
      end
      result.count.should == 6
      result.should include(:operation => 'create')
      result.should include(:object_type => 'index')
      result.should include(:index_name => 'pl_trigger_i2')
      result.should include(:table_name => 'pl_trigger')
      result.should include(:field => 'unique_no')
      result[:index_option].should include(:reverse_scans_property => 'allow reverse scans')

    end
    it 'live example' do
      sql = <<EOF
CREATE INDEX COST_JOB_AGR_1
        ON COST_JOB_AGR
                (JOB_ID)
EOF
      begin
        result = parser.parse(sql.downcase)
      rescue Parslet::ParseFailed => error
        puts error.cause.ascii_tree
      end
      result.count.should == 5
      result.should include(:operation => 'create')
      result.should include(:object_type => 'index')
      result.should include(:index_name => 'cost_job_agr_1')
      result.should include(:table_name => 'cost_job_agr')
      result.should include(:field => 'job_id')

    end
  end
end