require_relative '../../spec/spec_helper'

describe 'DDLParser::Parser' do
  
  it "knows if it's valid" do
    sql = 'CREATE TABLE TEST (ID INT)'
    parser = DDLParser::Parser.new(sql)
    parser.valid?.should == true
  end

  it "knows if it's invalid" do
    sql = 'CREATE TABLE TEST ()'
    parser = DDLParser::Parser.new(sql)
    parser.valid?.should == false
  end

  it 'can report errors' do
    sql = 'CREATE TABLE TEST ()'
    parser = DDLParser::Parser.new(sql)
    parser.errors.should include('CREATE_TABLE_STATEMENT')
  end

  it 'statement_type' do
    sql = 'CREATE TABLE TEST (ID INT, FOO INT)'
    parser = DDLParser::Parser.new(sql)
    parser.statement_type.should == :create_table
  end

  it 'returns the parse tree' do
    sql = 'CREATE TABLE TEST (ID INT, FOO INT)'
    parser = DDLParser::Parser.new(sql)
    parser.parse_tree.should == {:operation=> 'create table', :table_name=> 'test', :elements=>[
        {:column=>{:field=> 'id', :data_type=> 'int', :options=> ''}},
        {:column=>{:field=> 'foo', :data_type=> 'int', :options=> ''}}]}
  end

  context 'create table translate' do
    it 'translate table_name' do
      sql = 'CREATE TABLE TEST (ID INT, FOO INT)'
      parser = DDLParser::Parser.new(sql)
      parser.translate.table_name.should == 'test'
    end


    it 'translate columns' do
      sql = 'CREATE TABLE TEST (ID INT, FOO INT)'
      parser = DDLParser::Parser.new(sql)
      parser.translate.columns.map(&:to_hash).should == [{:field=> 'id', :data_type=> 'int', :options=> ''}, {:field=> 'foo', :data_type=> 'int', :options=> ''}]

      sql = 'CREATE TABLE TEST (ID INT)'
      parser = DDLParser::Parser.new(sql)
      parser.translate.columns.map(&:to_hash).should == [{:field=> 'id', :data_type=> 'int', :options=> ''}]
    end

    it 'translate primary key' do
      sql = 'CREATE TABLE TEST (ID   INT, PRIMARY KEY (ID))'
      parser = DDLParser::Parser.new(sql)
      parser.translate.primary_key.should == 'id'
    end
  end

  context 'alter table add column translate' do
    let(:sql) {
      sql = <<EOF
            ALTER TABLE BUDGET
              ADD COLUMN BUDGET_AMOUNT_IN_CO DECIMAL(15,2) NOT NULL DEFAULT 0
              ADD COLUMN BUDGET_AMOUNT_CURR CHARACTER(4)
EOF
      sql
    }

    it 'translate table_name' do
      parser = DDLParser::Parser.new(sql)
      parser.translate.table_name.should == 'budget'
    end


    it 'translate columns xxx' do
      parser = DDLParser::Parser.new(sql)
      parser.translate.add_columns.map(&:to_hash).should == [{:field=> 'budget_amount_in_co',
                                               :data_type=>{:decimal=>{:precision=>{:total=>{:integer=> '15'}, :scale=>{:integer=> '2'}}}},
                                               :options=>[{:column_option=> 'not null'},
                                                          {:default_clause=>[{:integer=> '0'}]}]},
                                              {:field=> 'budget_amount_curr',
                                               :data_type=>{:char=>{:length=>{:integer=> '4'}}},
                                               :options=> ''}]
    end

    it 'translate primary key' do
      sql = 'CREATE TABLE TEST (ID   INT, PRIMARY KEY (ID))'
      parser = DDLParser::Parser.new(sql)
      parser.translate.primary_key.should == 'id'
    end
  end

  context 'create index translate two index columns' do
    before :each do
      sql = 'CREATE UNIQUE INDEX REPLAN_ADDR_1 ON REPLAN_ADDR (CONSIGNMENT ASCENDING, COLLI DESCENDING)'
      @parser = DDLParser::Parser.new(sql)
    end

    it 'table name' do
      @parser.translate.table_name.should == 'replan_addr'
    end
    it 'index name' do
      @parser.translate.index_name.should == 'replan_addr_1'
    end
    it 'columns' do
      @parser.translate.columns.map(&:to_hash).should == [{:field=>"consignment", :sort_type=>"ascending"}, {:field=>"colli", :sort_type=>"descending"}]
    end

  end
  context 'create index translate one index column allow not reverse scans' do
    before :each do
      sql = 'CREATE UNIQUE INDEX REPLAN_ADDR_1 ON REPLAN_ADDR (CONSIGNMENT ASCENDING)'
      @parser = DDLParser::Parser.new(sql)
    end

    it 'table name' do
      @parser.translate.table_name.should == 'replan_addr'
    end
    it 'index name' do
      @parser.translate.index_name.should == 'replan_addr_1'
    end
    it 'columns' do
      @parser.translate.columns.map(&:to_hash).should == [{:field=>"consignment", :sort_type=>"ascending"}]
    end

    it 'allow not reverse scans 1' do
      @parser.translate.option_reverse_scans.should == 'N'
    end

  end
  context 'create index translate one index column allow reverse scans' do
    before :each do
      sql = 'CREATE UNIQUE INDEX REPLAN_ADDR_1 ON REPLAN_ADDR (CONSIGNMENT ASCENDING) ALLOW REVERSE SCANS'
      @parser = DDLParser::Parser.new(sql)
    end

    it 'table name' do
      @parser.translate.table_name.should == 'replan_addr'
    end
    it 'index name' do
      @parser.translate.index_name.should == 'replan_addr_1'
    end

    it 'allow reverse scans 1' do
      @parser.translate.option_reverse_scans.should == 'Y'
    end

  end
  context 'report errors' do
    it 'misses right parant' do
      sql = "Create table USER (FOO CHAR(4) NOT NULL, BAR CHAR(4) NOT NULL"
      parser = DDLParser::Parser.new(sql)
      parser.valid?.should == false
      parser.errors.should include("Premature end of input")
    end

    it 'misses element delimiter' do
      sql = "Create table USER (FOO CHAR(4) NOT NULL BAR CHAR(4) NOT NULL)"
      parser = DDLParser::Parser.new(sql)
      parser.valid?.should == false
      parser.errors.should include('Expected ")", but got "b"')
    end

    it 'must likely error 1' do
      sql = "Create table USER (FOO CHAR(4) NOT NULL, BAR CHAR(4) NOT NULL"
      parser = DDLParser::Parser.new(sql)
      parser.valid?.should == false
      parser.most_likely_error.should == 'Premature end of input at line 1 char 62.'
    end

    it 'must likely error 2' do
      sql = "Create table USER (FOO CHAR(4) NOT NULL BAR CHAR(4) NOT NULL)"
      parser = DDLParser::Parser.new(sql)
      parser.valid?.should == false
      parser.most_likely_error.should == 'Expected ")", but got "b" at line 1 char 41.'
    end

  end


end