require_relative '../../../../spec/spec_helper'

describe DDLParser::SQL::DB2::SelectParser do
  let(:parser) { DDLParser::SQL::DB2::SelectParser.new }

  context 'value parsing' do

    it 'parses integers' do
      expect(parser.const).to     parse('1')
      expect(parser.const).to     parse('-123')
      expect(parser.const).to     parse('120381')
      expect(parser.const).to     parse('181')
      expect(parser.const).to     parse('0181')
    end

    it 'parses floats' do
      expect(parser.const).to     parse('0.1')
      expect(parser.const).to     parse('3.14159')
      expect(parser.const).to     parse('-0.00001')
      expect(parser.const).to_not parse('.1')
    end

    it 'parses booleans' do
      expect(parser.const).to     parse('true')
      expect(parser.const).to     parse('false')
      expect(parser.const).to_not parse('truefalse')
    end

    it 'parses datetimes' do
      expect(parser.const).to     parse('1979-05-27T07:32:00Z')
      expect(parser.const).to     parse('2013-02-24T17:26:21Z')
      expect(parser.const).to_not parse('1979l05-27 07:32:00')
    end

    it 'parses strings' do
      expect(parser.const).to     parse("''")
      expect(parser.const).to     parse("'hello world'")
      expect(parser.const).to     parse("'120.0, ~!@#$\%^&*+_\}{ asdfasf'")
      expect(parser.const).not_to parse('"hello world"')
    end

  end

  context 'namelist parsing' do
    it 'simple' do
      expect(parser.namelist).to     parse('a,b,c')
      expect(parser.namelist).to     parse('a, b, C')
    end

    it 'not parse invalid' do
      expect(parser.namelist).not_to     parse('a b')
      expect(parser.namelist).not_to     parse('a: b')
      expect(parser.namelist).not_to     parse('a; b')
      expect(parser.namelist).not_to     parse("a, b, 'foo'" )
      expect(parser.namelist).not_to     parse('a, b, max(c)')
    end

  end

  context 'arglist parsing' do
    it 'simple' do
      expect(parser.arglist).to     parse('a,b,c')
      expect(parser.arglist).to     parse('a, b, c')
      expect(parser.arglist).to     parse("a, b, 'foo'" )
      expect(parser.arglist).to     parse('a, b, max(c)')
    end

    it 'not parse invalid' do
      expect(parser.arglist).not_to     parse('a b')
      expect(parser.arglist).not_to     parse('a: b')
      expect(parser.arglist).not_to     parse('a; b')
    end

  end

  context 'between parsing' do
    it 'should parse between' do
      expect(parser.single_cond).to   parse('a between 1 and 2')
      expect(parser.single_cond).to   parse('a between 1 and 2')
      expect(parser.single_cond.parse('a between 1 and 2')).to eq(:lhs=> 'a', :op=> 'between', :rhs=>{:btw_from=>{:integer=> '1'}, :btw_to=>{:integer=> '2'}})
    end
  end

  context 'like parsing' do
    it 'should parse' do
      expect(parser.single_cond).to   parse("a like '%foo%bar'")
    end

  end

  context 'condition parsing' do
    it 'should parse single' do
      expect(parser.condition).to   parse('a = a')
      expect(parser.condition).to   parse('a=a')
      expect(parser.condition).to   parse('a > a')
      expect(parser.condition).to   parse('a < a')
      expect(parser.condition).to   parse('a between 1 and 2')
      expect(parser.condition).to   parse("a like '%foo%bar'")
    end

    it 'should parse multible' do
      expect(parser.condition).to   parse('a = a or b = b')
      expect(parser.condition).to   parse('a = a and b = b')
      expect(parser.condition).to   parse("a > a and a between 'foo' and 'bar'")
      expect(parser.condition).to   parse("a < a and a like '%foo%bar'")
    end

  end

  context 'select statements' do
    it 'parse simple statements' do
      [
          'select a from b',
          'select a, b from comme',
          'select b from far',
          'select c from d',
          'select a, c, d,e,f,f from b ',
          'select count(a) from d',
      ].each do |sql|
        expect(parser).to parse(sql)
      end
    end

    it 'parse statements with where' do
      [
          'select a from b where a > 20',
          'select a,b from c where a = 20',
          'select a,b from c where 20 < a and b < 234',
          'select a,b from c where 20 < a or b = 234',
          'select a, b from c where a = ''oo''',
          'select a, b from c where a = ''oo'' and c > 235',
      ].each do |sql|
        expect(parser).to parse(sql)
      end
    end

    it 'parse statements with group by' do
      [
          'select a from b group by c',
          'select a from b where a > 345 group by c',
          'select a from b where a > 345 and foo = ''hoge'' group by c',
      ].each do |sql|
        expect(parser).to parse(sql)
      end
    end

    it 'not to parse bad selects' do
      [
        'select',
        'select a, b from c where a = "120.0, ~!@#$%^&*+_\}{\" asdfa"sf"',
      ].each do |sql|
        expect(parser).not_to parse(sql)
      end
    end

  end

  context 'extract' do
    it 'should fetch select' do
      result = parser.parse('select a from b where a = b')
      result[:select].should == {:item=> 'a'}
    end
  end


end
