require_relative '../../../spec/spec_helper'
require_relative '../../../lib/ddl_parser/translator/column'

describe 'DDLParser::Translator::Column' do

  let(:unique_no_column) {
    DDLParser::Translator::Column.new(
      {:field=> 'unique_no',
      :data_type=> 'integer',
      :options=>
        [{:column_option=> 'identity'},
        {:identity=>
          {:start_value=>{:integer=> '1'},
          :increment_value=>{:integer=> '1'},
          :cache_value=>{:integer=> '20'}
          }
        }]
      })
  }

  let(:int_column) {
    DDLParser::Translator::Column.new({:field=> 'int_field', :data_type=> 'int', :options=> ''})
  }

  let(:char_column) {
    DDLParser::Translator::Column.new(
      {:field=> 'char_field',
       :data_type=>{:char=>{:length=>{:integer=> '4'}}},
       :options=> [{:default_clause=>[{:string => "'foobar'"}]}]}
      )
  }


  let(:varchar_column) {
    DDLParser::Translator::Column.new(
        {:field=> 'varchar_field',
         :data_type=>{:varchar=>{:length=>{:integer=> '255'}}},
         :options=> [{:default_clause=>[{:string => "'foobar'"}]}]}
    )
  }

  let(:decimal_column) {
    DDLParser::Translator::Column.new(
      {:field=> 'decimal_field',
       :data_type=>{:decimal=>{:precision=>{:total=>{:integer=> '15'}, :scale=>{:integer=> '2'}}}},
       :options=>[{:column_option=> 'not null'}, {:default_clause=>[{:integer=> '0'}]}]
      })
  }

  let(:decimal_no_scale_column) {
    DDLParser::Translator::Column.new(
        {:field=> 'decimal_field',
         :data_type=>{:decimal=>{:precision=>{:total=>{:integer=> '15'}}}},
         :options=>[{:column_option=> 'not null'}, {:default_clause=>[{:integer=> '0'}]}]
        })
  }

  let(:decimal_no_precision_column) {
    DDLParser::Translator::Column.new(
        {:field=> 'decimal_field',
         :data_type=>{:decimal=>"decimal"},
         :options=>[{:column_option=> 'not null'}, {:default_clause=>[{:integer=> '0'}]}]
        })
  }



  it 'must receive hash' do
    DDLParser::Translator::Column.new('').to_hash.should == {}
    DDLParser::Translator::Column.new({:foo => :bar}).to_hash.should == {:foo => :bar}
  end

  it 'should return name' do
    int_column.name.should == 'int_field'
    char_column.name.should == 'char_field'
    decimal_column.name.should == 'decimal_field'
  end

  it 'should return data_type' do
    int_column.data_type.should == :int
    char_column.data_type.should == :char
    varchar_column.data_type.should == :varchar
    decimal_column.data_type.should == :decimal
  end

  it 'should return length' do
    int_column.length.should == nil
    char_column.length.should == 4
    varchar_column.length.should == 255
    decimal_column.length.should == 15.2
  end

  it 'should return precision' do
    int_column.precision.should == nil
    char_column.precision.should == nil
    decimal_column.precision.should == 15
    decimal_no_scale_column.precision.should == 15
    decimal_no_precision_column.precision.should == 5
  end

  it 'should return scale' do
    int_column.scale.should == nil
    char_column.scale.should == nil
    decimal_column.scale.should == 2
    decimal_no_scale_column.scale.should == 0
    decimal_no_precision_column.scale.should == 0
  end

  it 'should return options' do
    int_column.options.should == nil
    char_column.options.should == [{:default_clause=>[{:string =>"'foobar'"}]}]
    decimal_column.options.should == [{:column_option=> 'not null'}, {:default_clause=>[{:integer=> '0'}]}]
  end

it 'should return not null' do
  int_column.not_null.should == false
  char_column.not_null.should == false
  decimal_column.not_null.should == true
end

it 'should return default clause' do
  int_column.default_clause.should == nil
  char_column.default_clause.should == "'foobar'"
  decimal_column.default_clause.should == 0
end


it 'should return default value' do
  char_column.default_value.should == "'foobar'"
end

it 'should return identity flag' do
  unique_no_column.identity?.should == true
end

it 'should return start value for identity' do
  unique_no_column.start_value.should == 1
end


it 'should return start value 0 for int_column' do
  int_column.start_value.should == 0
end

it 'should return start value 0 for decimal_column' do
  decimal_column.start_value.should == 0
end

  it 'should return increment value for identity' do
  unique_no_column.increment_value.should == 1
end

it 'should return cache value for identity' do
  unique_no_column.cache_value.should == 20
end

it 'should return name via int_column_identity' do
  int_column_identity.name.should == 'identity_field'
end

it 'should return data_type via int_column_identity' do
  int_column_identity.data_type.should == :integer
end

it 'should return length via int_column_identity' do
  int_column_identity.length.should == nil
end

it 'should return precision via int_column_identity' do
  int_column_identity.precision.should == nil
end

it 'should return scale via int_column_identity' do
  int_column_identity.scale.should == nil
end

let(:int_column_identity) {
  DDLParser::Translator::Column.new(
      {:field=> 'identity_field',
       :data_type=>{:integer=> '1'},
       :options=>[{:column_option=> 'identity'}, {:identity_options=>[{:start_value=>[{:integer=> '1'},{:increment_value=>[{:integer=> '1'}]},{:cache_value=>[{:integer=> '20'}]}]}]
       }]})
}

it 'should return options via int_column_identity' do
  int_column_identity.options.should == [{:column_option=> 'identity'}, {:identity_options=>[{:start_value=>[{:integer=> '1'},{:increment_value=>[{:integer=> '1'}]},{:cache_value=>[{:integer=> '20'}]}]}]}]
end

it 'should return options alt via int_column_identity' do
  int_column_identity.identity_options.should == [{:integer=>'1'},{:increment_value=>[{:integer=>'1'}]},{:cache_value=>[{:integer=>'20'}]}]
end

end
